//+------------------------------------------------------------------+
//|                                              MT5_TetrisShape.mqh |
//|                      Copyright  2009, MetaQuotes Software Corp. |
//|                                        http://www.metaquotes.net |
//+------------------------------------------------------------------+
#property copyright "Copyright  2009, MetaQuotes Software Corp."
#property link      "http://www.metaquotes.net"

#include "MT5_TetrisShape.mqh"

#define WIDTH_IN_PIXELS  256
#define HEIGHT_IN_PIXELS 512
#define FIELD_WIDTH       16
#define FIELD_HEIGHT      32
#define HORZ_BORDER        6
#define VERT_BORDER        6

#define FIELD_NAME      "MT5TetrisField"
#define BACKGROUND_NAME "MT5TetrisBackGround"
#define SCORE_NAME      "MT5TetrisScore"
#define LABELOVER_NAME  "MT5TetrisOver"

//+------------------------------------------------------------------+
//|  -                                                   |
//+------------------------------------------------------------------+
class CTetrisField
  {
private:
   int               m_score;                            // 
   int               m_ypos;                             //       
   int               m_field[FIELD_HEIGHT][FIELD_WIDTH]; //  
   int               m_rows[FIELD_HEIGHT];               //       
   int               m_last_row;                         //    
   CTetrisShape     *m_shape;                            //  
   bool              m_bover;                            //  
public:
   void              CTetrisField() { m_shape=NULL; m_bover=false; }
   void              Init();
   void              Deinit();
   void              Down();
   void              Left();
   void              Right();
   void              Rotate();
   void              Drop();
private:
   void              NewShape();
   void              CheckAndDeleteRows();
   void              LabelOver();
  };
//+------------------------------------------------------------------+
//|              |
//+------------------------------------------------------------------+
void CTetrisField::Init()
  {
   int i;
//---       
   srand((uint)TimeCurrent());
//---  
   ArrayInitialize(m_field,0);
   for(i=0; i<FIELD_HEIGHT; i++)
      m_rows[i]=FIELD_HEIGHT-i-1;
   m_last_row=FIELD_HEIGHT;
//---   
   ObjectCreate(0,BACKGROUND_NAME,OBJ_BITMAP_LABEL,0,0,0,0,0);
   ObjectSetInteger(0,BACKGROUND_NAME,OBJPROP_XDISTANCE,2);
   ObjectSetInteger(0,BACKGROUND_NAME,OBJPROP_YDISTANCE,2);
   ObjectSetInteger(0,BACKGROUND_NAME,OBJPROP_BACK,true);
   ObjectSetString(0,BACKGROUND_NAME,OBJPROP_BMPFILE,"Tetris\\background.bmp");
//---   
   for(i=0; i<4; i++)
     {
      string name=SHAPE_NAME+(string)i;
      ObjectCreate(0,name,OBJ_BITMAP_LABEL,0,0,0,0,0);
      ObjectSetInteger(0,name,OBJPROP_BACK,false);
      ObjectSetString(0,name,OBJPROP_BMPFILE,"Tetris\\blue_block.bmp");
     }
//---      
   m_score=0;
//---   
   ObjectCreate(0,SCORE_NAME+"1",OBJ_LABEL,0,0,0,0,0);
   ObjectSetInteger(0,SCORE_NAME+"1",OBJPROP_XDISTANCE,WIDTH_IN_PIXELS/2+VERT_BORDER+3);
   ObjectSetInteger(0,SCORE_NAME+"1",OBJPROP_YDISTANCE,HEIGHT_IN_PIXELS+24+HORZ_BORDER+3);
   ObjectSetInteger(0,SCORE_NAME+"1",OBJPROP_BACK,false);
   ObjectSetInteger(0,SCORE_NAME+"1",OBJPROP_COLOR,Blue);
   ObjectSetString(0, SCORE_NAME+"1",OBJPROP_FONT,"Arial");
   ObjectSetInteger(0,SCORE_NAME+"1",OBJPROP_FONTSIZE,24);
   ObjectSetInteger(0,SCORE_NAME+"1",OBJPROP_ANCHOR,ANCHOR_TOP);
   ObjectSetDouble(0,SCORE_NAME+"1",OBJPROP_ANGLE,0.0);
   ObjectSetString(0,SCORE_NAME+"1",OBJPROP_TEXT,"0");
//---   
   ObjectCreate(0,SCORE_NAME,OBJ_LABEL,0,0,0,0,0);
   ObjectSetInteger(0,SCORE_NAME,OBJPROP_XDISTANCE,WIDTH_IN_PIXELS/2+VERT_BORDER);
   ObjectSetInteger(0,SCORE_NAME,OBJPROP_YDISTANCE,HEIGHT_IN_PIXELS+24+HORZ_BORDER);
   ObjectSetInteger(0,SCORE_NAME,OBJPROP_BACK,false);
   ObjectSetInteger(0,SCORE_NAME,OBJPROP_COLOR,Yellow);
   ObjectSetString(0, SCORE_NAME,OBJPROP_FONT,"Arial");
   ObjectSetInteger(0,SCORE_NAME,OBJPROP_FONTSIZE,24);
   ObjectSetInteger(0,SCORE_NAME,OBJPROP_ANCHOR,ANCHOR_TOP);
   ObjectSetDouble(0,SCORE_NAME,OBJPROP_ANGLE,0.0);
   ObjectSetString(0,SCORE_NAME,OBJPROP_TEXT,"0");
//---   
   NewShape();
  }
//+------------------------------------------------------------------+
//|                                                     |
//+------------------------------------------------------------------+
void CTetrisField::Deinit()
  {
   int    i,j,row;
   string name;
//---  
   ObjectDelete(0,BACKGROUND_NAME);
   ObjectDelete(0,SCORE_NAME);
   ObjectDelete(0,SCORE_NAME+"1");
   if(m_bover)
     {
      ObjectDelete(0,LABELOVER_NAME);
      ObjectDelete(0,LABELOVER_NAME+"1");
     }
//---  
   for(i=0; i<4; i++)
     {
      name=SHAPE_NAME+(string)i;
      ObjectDelete(0,name);
     }
//---  
   for(i=0; i<FIELD_HEIGHT; i++)
     {
      row=m_rows[i];
      for(j=0; j<FIELD_WIDTH; j++)
        {
         if(m_field[i][j]!=0)
           {
            name=FIELD_NAME+(string)row+"-"+(string)j;
            ObjectDelete(0,name);
           }
        }
     }
//---
  }
//+------------------------------------------------------------------+
//|                                                    |
//+------------------------------------------------------------------+
void CTetrisField::NewShape()
  {
   m_ypos=HORZ_BORDER;
//---      7  
   int nshape=rand()%7;
   switch(nshape)
     {
      case 0: m_shape=new CTetrisShape1; break;
      case 1: m_shape=new CTetrisShape2; break;
      case 2: m_shape=new CTetrisShape3; break;
      case 3: m_shape=new CTetrisShape4; break;
      case 4: m_shape=new CTetrisShape5; break;
      case 5: m_shape=new CTetrisShape6; break;
      case 6: m_shape=new CTetrisShape7; break;
     }
//--- 
   if(m_shape!=NULL)
     {
      //---  
      m_shape.SetRightBorder(WIDTH_IN_PIXELS+VERT_BORDER);
      m_shape.SetYPos(m_ypos);
      m_shape.SetXPos(VERT_BORDER+SHAPE_SIZE*8);
      //--- 
      m_shape.Draw();
     }
//---
  }
//+------------------------------------------------------------------+
//|                                                      |
//+------------------------------------------------------------------+
void CTetrisField::Down()
  {
   if(m_shape==NULL || m_bover) return;
//---
   int i,j,row,ypos,xpos;
   int x=(m_shape.GetXPos()-VERT_BORDER)/SHAPE_SIZE;
   int y=(m_shape.GetYPos()-HORZ_BORDER)/SHAPE_SIZE;
   int xsize=m_shape.GetXSize()/SHAPE_SIZE;
//---  
   int bottom[4];
   for(i=0; i<4; i++) bottom[i]=HEIGHT_IN_PIXELS+HORZ_BORDER;
//---     
   for(i=FIELD_HEIGHT-1; i>y; i--)
     {
      for(j=0; j<xsize; j++)
        {
         if(m_field[i][j+x]!=0) bottom[j]=i*SHAPE_SIZE;
        }
     }
//--- ,     
   if(!m_shape.CheckDown(bottom))
     {
      //---  ,  
      if(m_ypos==HORZ_BORDER)
        {
         m_bover=true;
         LabelOver();
        }
      //---
      int    type=m_shape.GetType();
      string name=SHAPE_NAME+"0";
      string bmp=ObjectGetString(0,name,OBJPROP_BMPFILE);
      string field_name;
      //---  4      
      for(i=0; i<4; i++)
        {
         name=SHAPE_NAME+(string)i;
         x=ObjectGetInteger(0,name,OBJPROP_XDISTANCE);
         y=ObjectGetInteger(0,name,OBJPROP_YDISTANCE);
         xpos=(x-VERT_BORDER)/SHAPE_SIZE;
         ypos=(y-HORZ_BORDER)/SHAPE_SIZE;
         row=m_rows[ypos];
         field_name=FIELD_NAME+(string)row+"-"+(string)xpos;
         //---   
         m_field[ypos][xpos]=type;
         //---      
         ObjectCreate(0,field_name,OBJ_BITMAP_LABEL,0,0,0,0,0);
         ObjectSetInteger(0,field_name,OBJPROP_BACK,false);
         ObjectSetString(0,field_name,OBJPROP_BMPFILE,bmp);
         ObjectSetInteger(0,field_name,OBJPROP_XDISTANCE,x);
         ObjectSetInteger(0,field_name,OBJPROP_YDISTANCE,y);
        }
      //---   
      delete m_shape;
      m_shape=NULL;
      //---   
      NewShape();
      //---    ,  
      CheckAndDeleteRows();
     }
   else
     {
      //---   
      m_ypos+=SHAPE_SIZE;
      m_shape.SetYPos(m_ypos);
      m_shape.Draw();
     }
//---
  }
//+------------------------------------------------------------------+
//|                                                     |
//+------------------------------------------------------------------+
void CTetrisField::Left()
  {
   if(m_shape==NULL || m_bover) return;
//---
   int i,j;
   int y=(m_shape.GetYPos()-HORZ_BORDER)/SHAPE_SIZE;
   int x=(m_shape.GetXPos()+m_shape.GetXSize()-VERT_BORDER)/SHAPE_SIZE;
   int ysize=m_shape.GetYSize()/SHAPE_SIZE;
//---    
   int left[4]={VERT_BORDER,VERT_BORDER,VERT_BORDER,VERT_BORDER};
//---     
   for(i=0; i<ysize; i++)
     {
      for(j=0; j<x; j++)
        {
         if(m_field[y+i][j]!=0)
            left[i]=(j+1)*SHAPE_SIZE;
        }
     }
//---    ,  
   if(m_shape.CheckLeft(left))
     {
      m_shape.Left();
      m_shape.Draw();
     }
//---
  }
//+------------------------------------------------------------------+
//|                                                    |
//+------------------------------------------------------------------+
void CTetrisField::Right()
  {
   if(m_shape==NULL || m_bover) return;
//---
   int i,j;
   int y=(m_shape.GetYPos()-HORZ_BORDER)/SHAPE_SIZE;
   int x=(m_shape.GetXPos()-VERT_BORDER)/SHAPE_SIZE;
   int ysize=m_shape.GetYSize()/SHAPE_SIZE;
//---    
   int right[4];
   for(i=0; i<4; i++) right[i]=WIDTH_IN_PIXELS+VERT_BORDER;
//---     
   for(i=0; i<ysize; i++)
     {
      for(j=FIELD_WIDTH-1; j>=x; j--)
        {
         if(m_field[y+i][j]!=0)
            right[i]=j*SHAPE_SIZE;
        }
     }
//---    ,  
   if(m_shape.CheckRight(right))
     {
      m_shape.Right();
      m_shape.Draw();
     }
//---
  }
//+------------------------------------------------------------------+
//|                                                     |
//+------------------------------------------------------------------+
void CTetrisField::Rotate()
  {
   if(m_shape==NULL || m_bover) return;
//---
   m_shape.Rotate();
   m_shape.Draw();
  }
//+------------------------------------------------------------------+
//|                                                      |
//+------------------------------------------------------------------+
void CTetrisField::Drop()
  {
   if(m_shape==NULL || m_bover) return;
//---
   int i,j;
   int x=(m_shape.GetXPos()-VERT_BORDER)/SHAPE_SIZE;
   int y=(m_shape.GetYPos()-HORZ_BORDER)/SHAPE_SIZE;
   int xsize=m_shape.GetXSize()/SHAPE_SIZE;
//---  
   int bottom[4];
   for(i=0; i<4; i++) bottom[i]=HEIGHT_IN_PIXELS+HORZ_BORDER;
//---     
   for(i=FIELD_HEIGHT-1; i>=0; i--)
     {
      for(j=0; j<xsize; j++)
        {
         if(m_field[i][j+x]!=0)
            bottom[j]=i*SHAPE_SIZE;
        }
     }
//--- ,     
   while(m_shape.CheckDown(bottom))
     {
      m_ypos+=SHAPE_SIZE;
      m_shape.SetYPos(m_ypos);
      m_shape.Draw();
     }
//---    
   Down();
//---
  }
//+------------------------------------------------------------------+
//|                                             |
//+------------------------------------------------------------------+
void CTetrisField::CheckAndDeleteRows()
  {
   int    i,j,x,y,score,row,to_delete=0;
   int    deleted[FIELD_HEIGHT];
   string name;
//---
   for(i=0; i<FIELD_HEIGHT; i++)
     {
      for(j=0; j<FIELD_WIDTH; j++)
        {
         if(m_field[i][j]==0) break;
        }
      //---   ,    
      if(j==FIELD_WIDTH)
         deleted[to_delete++]=i;
     }
//---   
   if(to_delete>0)
     {
      score=10;
      if(to_delete==2) score=30;
      if(to_delete==3) score=70;
      if(to_delete>3)  score*=to_delete*5;
      //---  
      for(i=0; i<to_delete; i++)
        {
         row=m_rows[deleted[i]];
         for(j=0; j<FIELD_WIDTH; j++)
           {
            name=FIELD_NAME+(string)row+"-"+(string)j;
            x=ObjectGetInteger(0,name,OBJPROP_XDISTANCE);
            y=ObjectGetInteger(0,name,OBJPROP_YDISTANCE);
            ObjectSetString(0,name,OBJPROP_BMPFILE,"Tetris\\delete_block.bmp");
            ObjectSetInteger(0,name,OBJPROP_XDISTANCE,x+3);
            ObjectSetInteger(0,name,OBJPROP_YDISTANCE,y+3);
           }
        }
      ChartRedraw();
      Sleep(500);
      //---  
      for(i=0; i<to_delete; i++)
        {
         row=m_rows[deleted[i]];
         for(j=0; j<FIELD_WIDTH; j++)
           {
            name=FIELD_NAME+(string)row+"-"+(string)j;
            ObjectDelete(0,name);
           }
        }
      ChartRedraw();
      //---        
      for(int d=0; d<to_delete; d++)
        {
         for(i=deleted[d]; i>0; i--)
           {
            row=m_rows[i-1];
            y=i*SHAPE_SIZE;
            for(j=0; j<FIELD_WIDTH; j++)
              {
               m_field[i][j]=m_field[i-1][j];
               name=FIELD_NAME+(string)row+"-"+(string)j;
               ObjectSetInteger(0,name,OBJPROP_YDISTANCE,y+HORZ_BORDER);
              }
            //---  ,     
            m_rows[i]=m_rows[i-1];
           }
         //---   
         for(j=0; j<FIELD_WIDTH; j++) m_field[0][j]=0;
         m_rows[0]=m_last_row++;
        }
      //---   
      m_score+=score;
      string strscore=string(m_score);
      ObjectSetString(0,SCORE_NAME+"1",OBJPROP_TEXT,strscore);
      ObjectSetString(0,SCORE_NAME,OBJPROP_TEXT,strscore);
     }
//---
  }
//+------------------------------------------------------------------+
//|  Game Over                                                |
//+------------------------------------------------------------------+
void CTetrisField::LabelOver()
  {
//---  
   ObjectCreate(0,LABELOVER_NAME+"1",OBJ_LABEL,0,0,0,0,0);
   ObjectSetInteger(0,LABELOVER_NAME+"1",OBJPROP_XDISTANCE,36+VERT_BORDER);
   ObjectSetInteger(0,LABELOVER_NAME+"1",OBJPROP_YDISTANCE,HEIGHT_IN_PIXELS/2+HORZ_BORDER);
   ObjectSetInteger(0,LABELOVER_NAME+"1",OBJPROP_BACK,false);
   ObjectSetInteger(0,LABELOVER_NAME+"1",OBJPROP_COLOR,Blue);
   ObjectSetString(0, LABELOVER_NAME+"1",OBJPROP_FONT,"Arial");
   ObjectSetInteger(0,LABELOVER_NAME+"1",OBJPROP_FONTSIZE,32);
   ObjectSetInteger(0,LABELOVER_NAME+"1",OBJPROP_ANCHOR,ANCHOR_LEFT_UPPER);
   ObjectSetDouble(0,LABELOVER_NAME+"1",OBJPROP_ANGLE,45.0);
   ObjectSetString(0,LABELOVER_NAME+"1",OBJPROP_TEXT,"Game Over");
//---  -  
   ObjectCreate(0,LABELOVER_NAME,OBJ_LABEL,0,0,0,0,0);
   ObjectSetInteger(0,LABELOVER_NAME,OBJPROP_XDISTANCE,30+VERT_BORDER);
   ObjectSetInteger(0,LABELOVER_NAME,OBJPROP_YDISTANCE,HEIGHT_IN_PIXELS/2+HORZ_BORDER);
   ObjectSetInteger(0,LABELOVER_NAME,OBJPROP_BACK,false);
   ObjectSetInteger(0,LABELOVER_NAME,OBJPROP_COLOR,Yellow);
   ObjectSetString(0, LABELOVER_NAME,OBJPROP_FONT,"Arial");
   ObjectSetInteger(0,LABELOVER_NAME,OBJPROP_FONTSIZE,32);
   ObjectSetInteger(0,LABELOVER_NAME,OBJPROP_ANCHOR,ANCHOR_LEFT_UPPER);
   ObjectSetDouble(0,LABELOVER_NAME,OBJPROP_ANGLE,45.0);
   ObjectSetString(0,LABELOVER_NAME,OBJPROP_TEXT,"Game Over");
//---
  }
//+------------------------------------------------------------------+
