//+------------------------------------------------------------------+
//|                                              MT5_TetrisShape.mqh |
//|                      Copyright  2009, MetaQuotes Software Corp. |
//|                                        http://www.metaquotes.net |
//+------------------------------------------------------------------+
#property copyright "Copyright  2009, MetaQuotes Software Corp."
#property link      "http://www.metaquotes.net"

#define SHAPE_NAME        "MT5TetrisShape"
#define SHAPE_SIZE        16

//+------------------------------------------------------------------+
//|                                         |
//+------------------------------------------------------------------+
class CTetrisShape
  {
protected:
   int               m_type;
   int               m_xpos;
   int               m_ypos;
   int               m_xsize;
   int               m_ysize;
   int               m_prev_turn;
   int               m_turn;
   int               m_right_border;
public:
   void              CTetrisShape();
   void              SetRightBorder(int border) { m_right_border=border; }
   void              SetYPos(int ypos)          { m_ypos=ypos;           }
   void              SetXPos(int xpos)          { m_xpos=xpos;           }
   int               GetYPos()                  { return(m_ypos);        }
   int               GetXPos()                  { return(m_xpos);        }
   int               GetYSize()                 { return(m_ysize);       }
   int               GetXSize()                 { return(m_xsize);       }
   int               GetType()                  { return(m_type);        }
   void              Left()                     { m_xpos-=SHAPE_SIZE;    }
   void              Right()                    { m_xpos+=SHAPE_SIZE;    }
   void              Rotate()                   { m_prev_turn=m_turn; if(++m_turn>3) m_turn=0; }
   virtual void      Draw()                     { return;                }
   virtual bool      CheckDown(int& pad_array[]);
   virtual bool      CheckLeft(int& side_row[]);
   virtual bool      CheckRight(int& side_row[]);
  };
//+------------------------------------------------------------------+
//|                                         |
//+------------------------------------------------------------------+
void CTetrisShape::CTetrisShape()
  {
   m_type=0;
   m_ypos=0;
   m_xpos=0;
   m_xsize=SHAPE_SIZE;
   m_ysize=SHAPE_SIZE;
   m_prev_turn=0;
   m_turn=0;
   m_right_border=0;
  }
//+------------------------------------------------------------------+
//|     (   )           |
//+------------------------------------------------------------------+
bool CTetrisShape::CheckDown(int& pad_array[])
  {
   int i,xsize=m_xsize/SHAPE_SIZE;
//---
   for(i=0; i<xsize; i++)
     {
      if(m_ypos+m_ysize>=pad_array[i]) return(false);
     }
//---
   return(true);
  }
//+------------------------------------------------------------------+
//|     (   )          |
//+------------------------------------------------------------------+
bool CTetrisShape::CheckLeft(int& side_row[])
  {
   int i,ysize=m_ysize/SHAPE_SIZE;
//---
   for(i=0; i<ysize; i++)
     {
      if(m_xpos<=side_row[i]) return(false);
     }
//---
   return(true);
  }
//+------------------------------------------------------------------+
//|     (   )         |
//+------------------------------------------------------------------+
bool CTetrisShape::CheckRight(int& side_row[])
  {
   int i,ysize=m_ysize/SHAPE_SIZE;
//---
   for(i=0; i<ysize; i++)
     {
      if(m_xpos+m_xsize>=side_row[i]) return(false);
     }
//---
   return(true);
  }
//+------------------------------------------------------------------+
//|                              |
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//|   4    ****                                        |
//+------------------------------------------------------------------+
class CTetrisShape1 : public CTetrisShape
  {
public:
   //--- 
   void              CTetrisShape1()
     {
      int    i;
      string name;
      //---
      for(i=0; i<4; i++)
        {
         name=SHAPE_NAME+(string)i;
         ObjectSetString(0,name,OBJPROP_BMPFILE,"Tetris\\red_block.bmp");
        }
      //---  
      m_xsize=SHAPE_SIZE*4;
      m_ysize=SHAPE_SIZE;
      m_type=1;
     }
   //---  
   virtual void      Draw()
     {
      int    i;
      string name;
      //---
      if(m_prev_turn!=m_turn)
        {
         //---   
         if(m_prev_turn==0 || m_prev_turn==2)
           {
            m_xsize=SHAPE_SIZE;
            m_ysize=SHAPE_SIZE*4;
            m_xpos+=SHAPE_SIZE;
            m_ypos-=SHAPE_SIZE;
            if(m_ypos<0) m_ypos=0;
           }
         else
           {
            m_xpos-=SHAPE_SIZE;
            if(m_xpos<0) m_xpos=0;
            m_ypos+=SHAPE_SIZE;
            m_xsize=SHAPE_SIZE*4;
            m_ysize=SHAPE_SIZE;
           }
         if(m_xpos+m_xsize>=m_right_border) m_xpos=m_right_border-m_xsize;
         m_prev_turn=m_turn;
        }
      if(m_turn==0 || m_turn==2)
        {
         //---  
         for(i=0; i<4; i++)
           {
            name=SHAPE_NAME+(string)i;
            ObjectSetInteger(0,name,OBJPROP_XDISTANCE,m_xpos+i*SHAPE_SIZE);
            ObjectSetInteger(0,name,OBJPROP_YDISTANCE,m_ypos);
           }
        }
      else
        {
         //---  
         for(i=0; i<4; i++)
           {
            name=SHAPE_NAME+(string)i;
            ObjectSetInteger(0,name,OBJPROP_XDISTANCE,m_xpos);
            ObjectSetInteger(0,name,OBJPROP_YDISTANCE,m_ypos+i*SHAPE_SIZE);
           }
        }
     }
  };
//+------------------------------------------------------------------+
//|     ***                                                    |
//|             *                                                    |
//+------------------------------------------------------------------+
class CTetrisShape2 : public CTetrisShape
  {
public:
   //--- 
   void              CTetrisShape2()
     {
      int    i;
      string name;
      //---
      for(i=0; i<4; i++)
        {
         name=SHAPE_NAME+(string)i;
         ObjectSetString(0,name,OBJPROP_BMPFILE,"Tetris\\blue_block.bmp");
        }
      //---   
      m_xsize=SHAPE_SIZE*3;
      m_ysize=SHAPE_SIZE*2;
      m_type=2;
     }
   //---     
   void              CalculateTurn()
     {
      if(m_prev_turn!=m_turn)
        {
         switch(m_prev_turn)
           {
            case 0:
            case 2:
               m_xsize=SHAPE_SIZE*2;
               m_ysize=SHAPE_SIZE*3;
               break;
            case 1:
            case 3:
               m_xsize=SHAPE_SIZE*3;
               m_ysize=SHAPE_SIZE*2;
               break;
           }
         if(m_xpos+m_xsize>=m_right_border) m_xpos=m_right_border-m_xsize;
         m_prev_turn=m_turn;
        }
     }
   //---    
   virtual void      CalculateBottom(int& bottom[])
     {
      switch(m_turn)
        {
         case 0:
            bottom[0]=m_ypos+SHAPE_SIZE;
            bottom[1]=m_ypos+SHAPE_SIZE;
            bottom[2]=m_ypos+2*SHAPE_SIZE;
            break;
         case 1:
            bottom[0]=m_ypos+3*SHAPE_SIZE;
            bottom[1]=m_ypos+3*SHAPE_SIZE;
            break;
         case 2:
            bottom[0]=m_ypos+2*SHAPE_SIZE;
            bottom[1]=m_ypos+2*SHAPE_SIZE;
            bottom[2]=m_ypos+2*SHAPE_SIZE;
            break;
         case 3:
            bottom[0]=m_ypos+3*SHAPE_SIZE;
            bottom[1]=m_ypos+SHAPE_SIZE;
            break;
        }
     }
   //---    
   virtual void      CalculateLeft(int& left[])
     {
      switch(m_turn)
        {
         case 0:
            left[0]=m_xpos;                 //  *** 
            left[1]=m_xpos+2*SHAPE_SIZE;    //    *
            break;
         case 1:
            left[0]=m_xpos+SHAPE_SIZE;      //   *
            left[1]=m_xpos+SHAPE_SIZE;      //   *
            left[2]=m_xpos;                 //  **
            break;
         case 2:
            left[0]=m_xpos;                 //  * 
            left[1]=m_xpos;                 //  ***
            break;
         case 3:
            left[0]=m_xpos;                 //  **
            left[1]=m_xpos;                 //  *
            left[2]=m_xpos;                 //  *
            break;
        }
     }
   //---    
   virtual void      CalculateRight(int& right[])
     {
      switch(m_turn)
        {
         case 0:
            right[0]=m_xpos+3*SHAPE_SIZE;   //  *** 
            right[1]=m_xpos+3*SHAPE_SIZE;   //    *
            break;
         case 1:
            right[0]=m_xpos+2*SHAPE_SIZE;   //   *
            right[1]=m_xpos+2*SHAPE_SIZE;   //   *
            right[2]=m_xpos+2*SHAPE_SIZE;   //  **
            break;
         case 2:
            right[0]=m_xpos+SHAPE_SIZE;     //  * 
            right[1]=m_xpos+3*SHAPE_SIZE;   //  ***
            break;
         case 3:
            right[0]=m_xpos+2*SHAPE_SIZE;   //  **
            right[1]=m_xpos+SHAPE_SIZE;     //  *
            right[2]=m_xpos+SHAPE_SIZE;     //  *
            break;
        }
     }
   //---  
   virtual void      Draw()
     {
      int    i;
      string name;
      //---
      CalculateTurn();
      switch(m_turn)
        {
         case 0:
            for(i=0; i<3; i++)
              {
               name=SHAPE_NAME+(string)i;
               ObjectSetInteger(0,name,OBJPROP_XDISTANCE,m_xpos+i*SHAPE_SIZE);
               ObjectSetInteger(0,name,OBJPROP_YDISTANCE,m_ypos);
              }
            name=SHAPE_NAME+"3";
            ObjectSetInteger(0,name,OBJPROP_XDISTANCE,m_xpos+2*SHAPE_SIZE);
            ObjectSetInteger(0,name,OBJPROP_YDISTANCE,m_ypos+SHAPE_SIZE);
            break;
         case 1:
            for(i=0; i<3; i++)
              {
               name=SHAPE_NAME+(string)i;
               ObjectSetInteger(0,name,OBJPROP_XDISTANCE,m_xpos+SHAPE_SIZE);
               ObjectSetInteger(0,name,OBJPROP_YDISTANCE,m_ypos+i*SHAPE_SIZE);
              }
            name=SHAPE_NAME+"3";
            ObjectSetInteger(0,name,OBJPROP_XDISTANCE,m_xpos);
            ObjectSetInteger(0,name,OBJPROP_YDISTANCE,m_ypos+2*SHAPE_SIZE);
            break;
         case 2:
            for(i=0; i<3; i++)
              {
               name=SHAPE_NAME+(string)i;
               ObjectSetInteger(0,name,OBJPROP_XDISTANCE,m_xpos+i*SHAPE_SIZE);
               ObjectSetInteger(0,name,OBJPROP_YDISTANCE,m_ypos+SHAPE_SIZE);
              }
            name=SHAPE_NAME+"3";
            ObjectSetInteger(0,name,OBJPROP_XDISTANCE,m_xpos);
            ObjectSetInteger(0,name,OBJPROP_YDISTANCE,m_ypos);
            break;
         case 3:
            for(i=0; i<3; i++)
              {
               name=SHAPE_NAME+(string)i;
               ObjectSetInteger(0,name,OBJPROP_XDISTANCE,m_xpos);
               ObjectSetInteger(0,name,OBJPROP_YDISTANCE,m_ypos+i*SHAPE_SIZE);
              }
            name=SHAPE_NAME+"3";
            ObjectSetInteger(0,name,OBJPROP_XDISTANCE,m_xpos+SHAPE_SIZE);
            ObjectSetInteger(0,name,OBJPROP_YDISTANCE,m_ypos);
            break;
        }
     }
   //---    
   virtual bool      CheckDown(int& pad_array[])
     {
      int i,xsize=m_xsize/SHAPE_SIZE;
      int bottom[4];
      //---     
      CalculateBottom(bottom);
      //---     
      for(i=0; i<xsize; i++)
        {
         if(bottom[i]>=pad_array[i]) return(false);
        }
      //---
      return(true);
     }
   //---    
   virtual bool      CheckLeft(int& side_row[])
     {
      int i,ysize=m_ysize/SHAPE_SIZE;
      int left[4];
      //---     
      CalculateLeft(left);
      //---     
      for(i=0; i<ysize; i++)
        {
         if(left[i]<=side_row[i]) return(false);
        }
      //---
      return(true);
     }
   //---    
   virtual bool      CheckRight(int& side_row[])
     {
      int i,ysize=m_ysize/SHAPE_SIZE;
      int right[4];
      //---     
      CalculateRight(right);
      //---     
      for(i=0; i<ysize; i++)
        {
         if(right[i]>=side_row[i]) return(false);
        }
      //---
      return(true);
     }
  };
//+------------------------------------------------------------------+
//|  T   ***                                                    |
//|            *                                                     |
//+------------------------------------------------------------------+
class CTetrisShape3 : public CTetrisShape2
  {
public:
   //--- 
   void              CTetrisShape3()
     {
      int    i;
      string name;
      //---
      for(i=0; i<4; i++)
        {
         name=SHAPE_NAME+(string)i;
         ObjectSetString(0,name,OBJPROP_BMPFILE,"Tetris\\yellow_block.bmp");
        }
      //---
      m_type=3;
     }
   //---    
   virtual void      CalculateBottom(int& bottom[])
     {
      switch(m_turn)
        {
         case 0:
            bottom[0]=m_ypos+SHAPE_SIZE;
            bottom[1]=m_ypos+2*SHAPE_SIZE;
            bottom[2]=m_ypos+SHAPE_SIZE;
            break;
         case 1:
            bottom[0]=m_ypos+2*SHAPE_SIZE;
            bottom[1]=m_ypos+3*SHAPE_SIZE;
            break;
         case 2:
            bottom[0]=m_ypos+2*SHAPE_SIZE;
            bottom[1]=m_ypos+2*SHAPE_SIZE;
            bottom[2]=m_ypos+2*SHAPE_SIZE;
            break;
         case 3:
            bottom[0]=m_ypos+3*SHAPE_SIZE;
            bottom[1]=m_ypos+2*SHAPE_SIZE;
            break;
        }
     }
   //---    
   virtual void      CalculateLeft(int& left[])
     {
      switch(m_turn)
        {
         case 0:
            left[0]=m_xpos;                 //  *** 
            left[1]=m_xpos+SHAPE_SIZE;      //   *
            break;
         case 1:
            left[0]=m_xpos+SHAPE_SIZE;      //   *
            left[1]=m_xpos;                 //  **
            left[2]=m_xpos+SHAPE_SIZE;      //   *
            break;
         case 2:
            left[0]=m_xpos+SHAPE_SIZE;      //   * 
            left[1]=m_xpos;                 //  ***
            break;
         case 3:
            left[0]=m_xpos;                 //  *
            left[1]=m_xpos;                 //  **
            left[2]=m_xpos;                 //  *
            break;
        }
     }
   //---    
   virtual void      CalculateRight(int& right[])
     {
      switch(m_turn)
        {
         case 0:
            right[0]=m_xpos+3*SHAPE_SIZE;   //  *** 
            right[1]=m_xpos+2*SHAPE_SIZE;   //   *
            break;
         case 1:
            right[0]=m_xpos+2*SHAPE_SIZE;   //   *
            right[1]=m_xpos+2*SHAPE_SIZE;   //  **
            right[2]=m_xpos+2*SHAPE_SIZE;   //   *
            break;
         case 2:
            right[0]=m_xpos+2*SHAPE_SIZE;   //   * 
            right[1]=m_xpos+3*SHAPE_SIZE;   //  ***
            break;
         case 3:
            right[0]=m_xpos+SHAPE_SIZE;     //  *
            right[1]=m_xpos+2*SHAPE_SIZE;   //  **
            right[2]=m_xpos+SHAPE_SIZE;     //  *
            break;
        }
     }
   //---  
   virtual void      Draw()
     {
      int    i;
      string name;
      //---
      CalculateTurn();
      switch(m_turn)
        {
         case 0:
            for(i=0; i<3; i++)
              {
               name=SHAPE_NAME+(string)i;
               ObjectSetInteger(0,name,OBJPROP_XDISTANCE,m_xpos+i*SHAPE_SIZE);
               ObjectSetInteger(0,name,OBJPROP_YDISTANCE,m_ypos);
              }
            name=SHAPE_NAME+"3";
            ObjectSetInteger(0,name,OBJPROP_XDISTANCE,m_xpos+SHAPE_SIZE);
            ObjectSetInteger(0,name,OBJPROP_YDISTANCE,m_ypos+SHAPE_SIZE);
            break;
         case 1:
            for(i=0; i<3; i++)
              {
               name=SHAPE_NAME+(string)i;
               ObjectSetInteger(0,name,OBJPROP_XDISTANCE,m_xpos+SHAPE_SIZE);
               ObjectSetInteger(0,name,OBJPROP_YDISTANCE,m_ypos+i*SHAPE_SIZE);
              }
            name=SHAPE_NAME+"3";
            ObjectSetInteger(0,name,OBJPROP_XDISTANCE,m_xpos);
            ObjectSetInteger(0,name,OBJPROP_YDISTANCE,m_ypos+SHAPE_SIZE);
            break;
         case 2:
            for(i=0; i<3; i++)
              {
               name=SHAPE_NAME+(string)i;
               ObjectSetInteger(0,name,OBJPROP_XDISTANCE,m_xpos+i*SHAPE_SIZE);
               ObjectSetInteger(0,name,OBJPROP_YDISTANCE,m_ypos+SHAPE_SIZE);
              }
            name=SHAPE_NAME+"3";
            ObjectSetInteger(0,name,OBJPROP_XDISTANCE,m_xpos+SHAPE_SIZE);
            ObjectSetInteger(0,name,OBJPROP_YDISTANCE,m_ypos);
            break;
         case 3:
            for(i=0; i<3; i++)
              {
               name=SHAPE_NAME+(string)i;
               ObjectSetInteger(0,name,OBJPROP_XDISTANCE,m_xpos);
               ObjectSetInteger(0,name,OBJPROP_YDISTANCE,m_ypos+i*SHAPE_SIZE);
              }
            name=SHAPE_NAME+"3";
            ObjectSetInteger(0,name,OBJPROP_XDISTANCE,m_xpos+SHAPE_SIZE);
            ObjectSetInteger(0,name,OBJPROP_YDISTANCE,m_ypos+SHAPE_SIZE);
            break;
        }
     }
  };
//+------------------------------------------------------------------+
//|  L   ***                                                    |
//|           *                                                      |
//+------------------------------------------------------------------+
class CTetrisShape4 : public CTetrisShape2
  {
public:
   //--- 
   void              CTetrisShape4()
     {
      int    i;
      string name;
      //---
      for(i=0; i<4; i++)
        {
         name=SHAPE_NAME+(string)i;
         ObjectSetString(0,name,OBJPROP_BMPFILE,"Tetris\\green_block.bmp");
        }
      ChartRedraw();
      //---
      m_type=4;
     }
   //---    
   virtual void      CalculateBottom(int& bottom[])
     {
      switch(m_turn)
        {
         case 0:
            bottom[0]=m_ypos+2*SHAPE_SIZE;
            bottom[1]=m_ypos+SHAPE_SIZE;
            bottom[2]=m_ypos+SHAPE_SIZE;
            break;
         case 1:
            bottom[0]=m_ypos+SHAPE_SIZE;
            bottom[1]=m_ypos+3*SHAPE_SIZE;
            break;
         case 2:
            bottom[0]=m_ypos+2*SHAPE_SIZE;
            bottom[1]=m_ypos+2*SHAPE_SIZE;
            bottom[2]=m_ypos+2*SHAPE_SIZE;
            break;
         case 3:
            bottom[0]=m_ypos+3*SHAPE_SIZE;
            bottom[1]=m_ypos+3*SHAPE_SIZE;
            break;
        }
     }
   //---    
   virtual void      CalculateLeft(int& left[])
     {
      switch(m_turn)
        {
         case 0:
            left[0]=m_xpos;                 //  *** 
            left[1]=m_xpos;                 //  *
            break;
         case 1:
            left[0]=m_xpos;                 //  **
            left[1]=m_xpos+SHAPE_SIZE;      //   *
            left[2]=m_xpos+SHAPE_SIZE;      //   *
            break;
         case 2:
            left[0]=m_xpos+2*SHAPE_SIZE;    //    * 
            left[1]=m_xpos;                 //  ***
            break;
         case 3:
            left[0]=m_xpos;                 //  *
            left[1]=m_xpos;                 //  *
            left[2]=m_xpos;                 //  **
            break;
        }
     }
   //---    
   virtual void      CalculateRight(int& right[])
     {
      switch(m_turn)
        {
         case 0:
            right[0]=m_xpos+3*SHAPE_SIZE;   //  *** 
            right[1]=m_xpos+SHAPE_SIZE;     //  *
            break;
         case 1:
            right[0]=m_xpos+2*SHAPE_SIZE;   //  **
            right[1]=m_xpos+2*SHAPE_SIZE;   //   *
            right[2]=m_xpos+2*SHAPE_SIZE;   //   *
            break;
         case 2:
            right[0]=m_xpos+3*SHAPE_SIZE;   //    * 
            right[1]=m_xpos+3*SHAPE_SIZE;   //  ***
            break;
         case 3:
            right[0]=m_xpos+SHAPE_SIZE;     //  *
            right[1]=m_xpos+SHAPE_SIZE;     //  *
            right[2]=m_xpos+2*SHAPE_SIZE;   //  **
            break;
        }
     }
   //---  
   virtual void      Draw()
     {
      int    i;
      string name;
      //---
      CalculateTurn();
      switch(m_turn)
        {
         case 0:
            for(i=0; i<3; i++)
              {
               name=SHAPE_NAME+(string)i;
               ObjectSetInteger(0,name,OBJPROP_XDISTANCE,m_xpos+i*SHAPE_SIZE);
               ObjectSetInteger(0,name,OBJPROP_YDISTANCE,m_ypos);
              }
            name=SHAPE_NAME+"3";
            ObjectSetInteger(0,name,OBJPROP_XDISTANCE,m_xpos);
            ObjectSetInteger(0,name,OBJPROP_YDISTANCE,m_ypos+SHAPE_SIZE);
            break;
         case 1:
            for(i=0; i<3; i++)
              {
               name=SHAPE_NAME+(string)i;
               ObjectSetInteger(0,name,OBJPROP_XDISTANCE,m_xpos+SHAPE_SIZE);
               ObjectSetInteger(0,name,OBJPROP_YDISTANCE,m_ypos+i*SHAPE_SIZE);
              }
            name=SHAPE_NAME+"3";
            ObjectSetInteger(0,name,OBJPROP_XDISTANCE,m_xpos);
            ObjectSetInteger(0,name,OBJPROP_YDISTANCE,m_ypos);
            break;
         case 2:
            for(i=0; i<3; i++)
              {
               name=SHAPE_NAME+(string)i;
               ObjectSetInteger(0,name,OBJPROP_XDISTANCE,m_xpos+i*SHAPE_SIZE);
               ObjectSetInteger(0,name,OBJPROP_YDISTANCE,m_ypos+SHAPE_SIZE);
              }
            name=SHAPE_NAME+"3";
            ObjectSetInteger(0,name,OBJPROP_XDISTANCE,m_xpos+2*SHAPE_SIZE);
            ObjectSetInteger(0,name,OBJPROP_YDISTANCE,m_ypos);
            break;
         case 3:
            for(i=0; i<3; i++)
              {
               name=SHAPE_NAME+(string)i;
               ObjectSetInteger(0,name,OBJPROP_XDISTANCE,m_xpos);
               ObjectSetInteger(0,name,OBJPROP_YDISTANCE,m_ypos+i*SHAPE_SIZE);
              }
            name=SHAPE_NAME+"3";
            ObjectSetInteger(0,name,OBJPROP_XDISTANCE,m_xpos+SHAPE_SIZE);
            ObjectSetInteger(0,name,OBJPROP_YDISTANCE,m_ypos+2*SHAPE_SIZE);
            break;
        }
     }
  };
//+------------------------------------------------------------------+
//|     **                                                    |
//|             **                                                   |
//+------------------------------------------------------------------+
class CTetrisShape5 : public CTetrisShape2
  {
public:
   //--- 
   void              CTetrisShape5()
     {
      int    i;
      string name;
      //---
      for(i=0; i<4; i++)
        {
         name=SHAPE_NAME+(string)i;
         ObjectSetString(0,name,OBJPROP_BMPFILE,"Tetris\\magenta_block.bmp");
        }
      //---
      m_type=5;
     }
   //---    
   virtual void      CalculateBottom(int& bottom[])
     {
      switch(m_turn)
        {
         case 0:
         case 2:
            bottom[0]=m_ypos+SHAPE_SIZE;
            bottom[1]=m_ypos+2*SHAPE_SIZE;
            bottom[2]=m_ypos+2*SHAPE_SIZE;
            break;
         case 1:
         case 3:
            bottom[0]=m_ypos+3*SHAPE_SIZE;
            bottom[1]=m_ypos+2*SHAPE_SIZE;
            break;
        }
     }
   //---    
   virtual void      CalculateLeft(int& left[])
     {
      switch(m_turn)
        {
         case 0:
         case 2:
            left[0]=m_xpos;                 //  ** 
            left[1]=m_xpos+SHAPE_SIZE;      //   **
            break;
         case 1:
         case 3:
            left[0]=m_xpos+SHAPE_SIZE;      //   *
            left[1]=m_xpos;                 //  **
            left[2]=m_xpos;                 //  *
            break;
        }
     }
   //---    
   virtual void      CalculateRight(int& right[])
     {
      switch(m_turn)
        {
         case 0:
         case 2:
            right[0]=m_xpos+2*SHAPE_SIZE;   //  ** 
            right[1]=m_xpos+3*SHAPE_SIZE;   //   **
            break;
         case 1:
         case 3:
            right[0]=m_xpos+2*SHAPE_SIZE;   //   *
            right[1]=m_xpos+2*SHAPE_SIZE;   //  **
            right[2]=m_xpos+SHAPE_SIZE;     //  *
            break;
        }
     }
   //---  
   virtual void      Draw()
     {
      int    i;
      string name;
      //---
      CalculateTurn();
      switch(m_turn)
        {
         case 0:
         case 2:
            for(i=0; i<2; i++)
              {
               name=SHAPE_NAME+(string)i;
               ObjectSetInteger(0,name,OBJPROP_XDISTANCE,m_xpos+i*SHAPE_SIZE);
               ObjectSetInteger(0,name,OBJPROP_YDISTANCE,m_ypos);
              }
            for(i=2; i<4; i++)
              {
               name=SHAPE_NAME+(string)i;
               ObjectSetInteger(0,name,OBJPROP_XDISTANCE,m_xpos+(i-1)*SHAPE_SIZE);
               ObjectSetInteger(0,name,OBJPROP_YDISTANCE,m_ypos+SHAPE_SIZE);
              }
            break;
         case 1:
         case 3:
            for(i=0; i<2; i++)
              {
               name=SHAPE_NAME+(string)i;
               ObjectSetInteger(0,name,OBJPROP_XDISTANCE,m_xpos);
               ObjectSetInteger(0,name,OBJPROP_YDISTANCE,m_ypos+(i+1)*SHAPE_SIZE);
              }
            for(i=2; i<4; i++)
              {
               name=SHAPE_NAME+(string)i;
               ObjectSetInteger(0,name,OBJPROP_XDISTANCE,m_xpos+SHAPE_SIZE);
               ObjectSetInteger(0,name,OBJPROP_YDISTANCE,m_ypos+(i-2)*SHAPE_SIZE);
              }
            break;
        }
     }
  };
//+------------------------------------------------------------------+
//|   4    **                                          |
//|                      **                                          |
//+------------------------------------------------------------------+
class CTetrisShape6 : public CTetrisShape
  {
public:
   //--- 
   void              CTetrisShape6()
     {
      int    i;
      string name;
      //---
      for(i=0; i<4; i++)
        {
         name=SHAPE_NAME+(string)i;
         ObjectSetString(0,name,OBJPROP_BMPFILE,"Tetris\\orange_block.bmp");
        }
      //---  
      m_xsize=SHAPE_SIZE*2;
      m_ysize=SHAPE_SIZE*2;
      m_type=6;
     }
   //---  
   virtual void      Draw()
     {
      int    i;
      string name;
      //---
      for(i=0; i<2; i++)
        {
         name=SHAPE_NAME+(string)i;
         ObjectSetInteger(0,name,OBJPROP_XDISTANCE,m_xpos+i*SHAPE_SIZE);
         ObjectSetInteger(0,name,OBJPROP_YDISTANCE,m_ypos);
        }
      for(i=2; i<4; i++)
        {
         name=SHAPE_NAME+(string)i;
         ObjectSetInteger(0,name,OBJPROP_XDISTANCE,m_xpos+(i-2)*SHAPE_SIZE);
         ObjectSetInteger(0,name,OBJPROP_YDISTANCE,m_ypos+SHAPE_SIZE);
        }
     }
  };
//+------------------------------------------------------------------+
//|      **                                                   |
//|            **                                                    |
//+------------------------------------------------------------------+
class CTetrisShape7 : public CTetrisShape2
  {
public:
   //--- 
   void              CTetrisShape7()
     {
      int    i;
      string name;
      //---
      for(i=0; i<4; i++)
        {
         name=SHAPE_NAME+(string)i;
         ObjectSetString(0,name,OBJPROP_BMPFILE,"Tetris\\brown_block.bmp");
        }
      //---
      m_type=7;
     }
   //---    
   virtual void      CalculateBottom(int& bottom[])
     {
      switch(m_turn)
        {
         case 0:
         case 2:
            bottom[0]=m_ypos+2*SHAPE_SIZE;
            bottom[1]=m_ypos+2*SHAPE_SIZE;
            bottom[2]=m_ypos+SHAPE_SIZE;
            break;
         case 1:
         case 3:
            bottom[0]=m_ypos+2*SHAPE_SIZE;
            bottom[1]=m_ypos+3*SHAPE_SIZE;
            break;
        }
     }
   //---    
   virtual void      CalculateLeft(int& left[])
     {
      switch(m_turn)
        {
         case 0:
         case 2:
            left[0]=m_xpos+SHAPE_SIZE;      //   **
            left[1]=m_xpos;                 //  ** 
            break;
         case 1:
         case 3:
            left[0]=m_xpos;                 //  *
            left[1]=m_xpos;                 //  **
            left[2]=m_xpos+SHAPE_SIZE;      //   *
            break;
        }
     }
   //---    
   virtual void      CalculateRight(int& right[])
     {
      switch(m_turn)
        {
         case 0:
         case 2:
            right[0]=m_xpos+3*SHAPE_SIZE;   //   **
            right[1]=m_xpos+2*SHAPE_SIZE;   //  ** 
            break;
         case 1:
         case 3:
            right[0]=m_xpos+SHAPE_SIZE;     //  *
            right[1]=m_xpos+2*SHAPE_SIZE;   //  **
            right[2]=m_xpos+2*SHAPE_SIZE;   //   *
            break;
        }
     }
   //---  
   virtual void      Draw()
     {
      int    i;
      string name;
      //---
      CalculateTurn();
      switch(m_turn)
        {
         case 0:
         case 2:
            for(i=0; i<2; i++)
              {
               name=SHAPE_NAME+(string)i;
               ObjectSetInteger(0,name,OBJPROP_XDISTANCE,m_xpos+(i+1)*SHAPE_SIZE);
               ObjectSetInteger(0,name,OBJPROP_YDISTANCE,m_ypos);
              }
            for(i=2; i<4; i++)
              {
               name=SHAPE_NAME+(string)i;
               ObjectSetInteger(0,name,OBJPROP_XDISTANCE,m_xpos+(i-2)*SHAPE_SIZE);
               ObjectSetInteger(0,name,OBJPROP_YDISTANCE,m_ypos+SHAPE_SIZE);
              }
            break;
         case 1:
         case 3:
            for(i=0; i<2; i++)
              {
               name=SHAPE_NAME+(string)i;
               ObjectSetInteger(0,name,OBJPROP_XDISTANCE,m_xpos);
               ObjectSetInteger(0,name,OBJPROP_YDISTANCE,m_ypos+i*SHAPE_SIZE);
              }
            for(i=2; i<4; i++)
              {
               name=SHAPE_NAME+(string)i;
               ObjectSetInteger(0,name,OBJPROP_XDISTANCE,m_xpos+SHAPE_SIZE);
               ObjectSetInteger(0,name,OBJPROP_YDISTANCE,m_ypos+(i-1)*SHAPE_SIZE);
              }
            break;
        }
     }
  };
//+------------------------------------------------------------------+
